package de.willemer.bermuda;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.TextView;

public class BermudaView extends View
        implements View.OnTouchListener {

    private Paint paint = new Paint();
    private Bitmap leer, schiff, ziffer[];
    private int xDist=1, yDist=1;

    private Spielfeld spiel = new Spielfeld();
    private TextView messageTV = null;
    private int score = 0;

    public BermudaView(Context context,
                       @Nullable AttributeSet attrs) {
        super(context, attrs);
        this.setOnTouchListener(this);
        leer = BitmapFactory.decodeResource(getResources(),
                R.drawable.leer);
        schiff = BitmapFactory.decodeResource(getResources(),
                R.drawable.schiff);
        ziffer = new Bitmap[9];
        for (int i=0; i<9; i++) {
            ziffer[i] = BitmapFactory.decodeResource(
                    getResources(),
                    getResources().getIdentifier(
                        "mark"+i, "drawable",
                        context.getPackageName()));
        }
        xDist = schiff.getWidth();
        yDist = schiff.getHeight();
    }

    @Override
    public void onDraw(Canvas canvas) {
        canvas.drawColor(Color.WHITE);
        for (int x=0; x<Spielfeld.MAXX; x++) {
            for (int y=0; y<Spielfeld.MAXY; y++) {
                Position pos = new Position(x, y);
                char c = spiel.get(pos);
                Bitmap img = null;
                if (c >= '0' && c <= '8') {
                    img = ziffer[c-'0'];
                } else if (c == Spielfeld.SCHIFF) {
                    img = schiff;
                } else {
                    img = leer;
                }
                canvas.drawBitmap(img, x*xDist, y*yDist, paint);
            }
        }
    }

    @Override
    public boolean onTouch(View view, MotionEvent me) {
        int suchX = (int)me.getX()/xDist;
        int suchY = (int)me.getY()/yDist;
        if (suchX>=0 && suchY>=0 && suchX<Spielfeld.MAXX
                     && suchY<Spielfeld.MAXY) {
            Position pos = new Position(suchX, suchY);
            char c = spiel.suche(pos);
            spiel.set(pos, c);
            score++;
            String str = getResources().getString(
                    R.string.versuche);
            str = "" + score + " " + str + " ";
            messageTV.setText(str);
            if (c==Spielfeld.SCHIFF) {
                if (spiel.alleSchiffeGefunden()) {
                    str += getResources().getString(
                            R.string.won);
                    messageTV.setText(str);
                    messageTV.setBackgroundColor(Color.RED);
                }
            }
            invalidate();
        }
        return false;
    }

    public void neuesSpiel() {
        spiel = new Spielfeld();
        String str = getResources().getString(
                R.string.message);
        messageTV.setText(str);
        messageTV.setBackgroundColor(Color.LTGRAY);
        score = 0;
        invalidate();
    }

    public void transferMessageTV(TextView tv) {
        messageTV = tv;
    }
}
