package gast;

import java.net.URI;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

@Path("gast")
public class GastServer {

	@GET
	@Path("all")
	@Produces(MediaType.APPLICATION_XML)
	public List<Gast> getAllXml() {
		System.out.println("getAll");
		List<Gast> list = GastDao.getInstance().getGast("");
		return list;
	}

	@GET
	@Produces(MediaType.APPLICATION_XML)
	@Path("id/{id}")
	public Gast getOneXml(@PathParam("id") long id) {
		System.out.println("getONe");
		Gast gast = GastDao.getInstance().getGast(id);
		if (gast == null)
			throw new NotFoundException();
		return gast;
	}

	@GET
	@Produces(MediaType.APPLICATION_XML)
	@Path("name/{name}")
	public List<Gast> getSomeXml(@PathParam("name") String name) {
		List<Gast> liste = GastDao.getInstance().getGast(name);
		if (liste.isEmpty()) {
			// throw new NotFoundException();
		}
		return liste;
	}
	
	@PUT
	@Consumes(MediaType.APPLICATION_XML)
	@Produces(MediaType.APPLICATION_XML)
	public Response putGast(Gast gast) {
		long pos = GastDao.getInstance().addGast(gast);
		if (pos < 0) {
			throw new WebApplicationException(Response.status(400)
					.entity("Not created.").build());
		}
		gast.setId(pos); // uebernehme die erzeugte ID
		URI uri = uriInfo.getAbsolutePathBuilder().path(Long.toString(pos)).build();
        return Response.created(uri).entity(gast).build();
	}
	
	@GET
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	public String getNameHtml(@QueryParam("name") String name) {
		String str = "<html><body><h1>G&auml;steliste</h1>";
		str += "<table br=1>";
		if (name == null) {
			name = "";
		}
		List<Gast> liste = GastDao.getInstance().getGast(name);
		for (Gast gast : liste) {
			str += "<tr><td>" + gast.getId();
			str += "</td><td>" + gast.getName();
			str += "</td><td>" + gast.getAdresse();
			str += "</td><td>" + gast.getOrt();
			str += "</td><td>" + gast.getEmail();
			str += "</td><td>" + gast.getTelefon();
			str += "</td></tr>";
		}
		str += "</table></body></html>";
		return str;
	}
	
	@DELETE
	@Path("id/{id}")
	public Response deleteGast(@PathParam("id") Long id) {
	    if (GastDao.getInstance().deleteGast(id)) {
	        return Response.accepted().build();
	    }
	    throw new NotFoundException();
	}

	
	// Wird von den POST-Methoden benötigt, um weiterzuleiten
	@Context
	UriInfo uriInfo;

	@POST
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	// Für die Erzeugung über ein Browserformular
	public Response postHtml(@FormParam("id") String id, @FormParam("name") String name,
			@FormParam("adresse") String adresse, @FormParam("ort") String ort, @FormParam("email") String email,
			@FormParam("telefon") String telefon) {
		if (name == null) {
			throw new WebApplicationException(Response.status(400).entity("Could not be created.").build());
		}
		Gast gast = new Gast();
		gast.setName(name);
		gast.setAdresse(adresse);
		gast.setOrt(ort);
		gast.setEmail(email);
		gast.setTelefon(telefon);
		long pos = -1;
		if (id != null && !id.isEmpty()) {
			gast.setId(Integer.parseInt(id));
			pos = GastDao.getInstance().changeGast(gast);
		} else {
			// Keine id, also wird erzeugt!
			pos = GastDao.getInstance().addGast(gast);
		}
		URI location = uriInfo.getAbsolutePathBuilder().path("" + pos).build();
		return Response.created(location).build();
	}

}
